import { __DEBUG_BUILD__ } from "../../../types/src/globals.ts";
import {
  Event,
  EventProcessor,
  Hub,
  Integration,
  StackFrame,
} from "../../../types/src/index.ts";
import {
  getEventDescription,
  isMatchingPattern,
  logger,
} from "../../../utils/src/index.ts";

// "Script error." is hard coded into browsers for errors that it can't read.
// this is the result of a script being pulled in from an external domain and CORS.
const DEFAULT_IGNORE_ERRORS = [
  /^Script error\.?$/,
  /^Javascript error: Script error\.? on line 0$/,
];

/** Options for the InboundFilters integration */
export interface InboundFiltersOptions {
  allowUrls: Array<string | RegExp>;
  denyUrls: Array<string | RegExp>;
  ignoreErrors: Array<string | RegExp>;
  ignoreInternal: boolean;
}

/** Inbound filters configurable by the user */
export class InboundFilters implements Integration {
  /**
   * @inheritDoc
   */
  public static id: string = "InboundFilters";

  /**
   * @inheritDoc
   */
  public name: string = InboundFilters.id;

  public constructor(
    private readonly _options: Partial<InboundFiltersOptions> = {},
  ) {}

  /**
   * @inheritDoc
   */
  public setupOnce(
    addGlobalEventProcessor: (processor: EventProcessor) => void,
    getCurrentHub: () => Hub,
  ): void {
    const eventProcess: EventProcessor = (event: Event) => {
      const hub = getCurrentHub();
      if (hub) {
        const self = hub.getIntegration(InboundFilters);
        if (self) {
          const client = hub.getClient();
          const clientOptions = client ? client.getOptions() : {};
          const options = _mergeOptions(self._options, clientOptions);
          return _shouldDropEvent(event, options) ? null : event;
        }
      }
      return event;
    };

    eventProcess.id = this.name;
    addGlobalEventProcessor(eventProcess);
  }
}

/** JSDoc */
export function _mergeOptions(
  internalOptions: Partial<InboundFiltersOptions> = {},
  clientOptions: Partial<InboundFiltersOptions> = {},
): Partial<InboundFiltersOptions> {
  return {
    allowUrls: [
      ...(internalOptions.allowUrls || []),
      ...(clientOptions.allowUrls || []),
    ],
    denyUrls: [
      ...(internalOptions.denyUrls || []),
      ...(clientOptions.denyUrls || []),
    ],
    ignoreErrors: [
      ...(internalOptions.ignoreErrors || []),
      ...(clientOptions.ignoreErrors || []),
      ...DEFAULT_IGNORE_ERRORS,
    ],
    ignoreInternal: internalOptions.ignoreInternal !== undefined
      ? internalOptions.ignoreInternal
      : true,
  };
}

/** JSDoc */
export function _shouldDropEvent(
  event: Event,
  options: Partial<InboundFiltersOptions>,
): boolean {
  if (options.ignoreInternal && _isSentryError(event)) {
    __DEBUG_BUILD__ &&
      logger.warn(
        `Event dropped due to being internal Sentry Error.\nEvent: ${
          getEventDescription(event)
        }`,
      );
    return true;
  }
  if (_isIgnoredError(event, options.ignoreErrors)) {
    __DEBUG_BUILD__ &&
      logger.warn(
        `Event dropped due to being matched by \`ignoreErrors\` option.\nEvent: ${
          getEventDescription(event)
        }`,
      );
    return true;
  }
  if (_isDeniedUrl(event, options.denyUrls)) {
    __DEBUG_BUILD__ &&
      logger.warn(
        `Event dropped due to being matched by \`denyUrls\` option.\nEvent: ${
          getEventDescription(
            event,
          )
        }.\nUrl: ${_getEventFilterUrl(event)}`,
      );
    return true;
  }
  if (!_isAllowedUrl(event, options.allowUrls)) {
    __DEBUG_BUILD__ &&
      logger.warn(
        `Event dropped due to not being matched by \`allowUrls\` option.\nEvent: ${
          getEventDescription(
            event,
          )
        }.\nUrl: ${_getEventFilterUrl(event)}`,
      );
    return true;
  }
  return false;
}

function _isIgnoredError(
  event: Event,
  ignoreErrors?: Array<string | RegExp>,
): boolean {
  if (!ignoreErrors || !ignoreErrors.length) {
    return false;
  }

  return _getPossibleEventMessages(event).some((message) =>
    ignoreErrors.some((pattern) => isMatchingPattern(message, pattern))
  );
}

function _isDeniedUrl(
  event: Event,
  denyUrls?: Array<string | RegExp>,
): boolean {
  // TODO: Use Glob instead?
  if (!denyUrls || !denyUrls.length) {
    return false;
  }
  const url = _getEventFilterUrl(event);
  return !url
    ? false
    : denyUrls.some((pattern) => isMatchingPattern(url, pattern));
}

function _isAllowedUrl(
  event: Event,
  allowUrls?: Array<string | RegExp>,
): boolean {
  // TODO: Use Glob instead?
  if (!allowUrls || !allowUrls.length) {
    return true;
  }
  const url = _getEventFilterUrl(event);
  return !url
    ? true
    : allowUrls.some((pattern) => isMatchingPattern(url, pattern));
}

function _getPossibleEventMessages(event: Event): string[] {
  if (event.message) {
    return [event.message];
  }
  if (event.exception) {
    try {
      const { type = "", value = "" } =
        (event.exception.values && event.exception.values[0]) || {};
      return [`${value}`, `${type}: ${value}`];
    } catch (oO) {
      __DEBUG_BUILD__ &&
        logger.error(
          `Cannot extract message for event ${getEventDescription(event)}`,
        );
      return [];
    }
  }
  return [];
}

function _isSentryError(event: Event): boolean {
  try {
    // @ts-ignore can't be a sentry error if undefined
    // eslint-disable-next-line @typescript-eslint/no-unsafe-member-access
    return event.exception.values[0].type === "SentryError";
  } catch (e) {
    // ignore
  }
  return false;
}

function _getLastValidUrl(frames: StackFrame[] = []): string | null {
  for (let i = frames.length - 1; i >= 0; i--) {
    const frame = frames[i];

    if (
      frame && frame.filename !== "<anonymous>" &&
      frame.filename !== "[native code]"
    ) {
      return frame.filename || null;
    }
  }

  return null;
}

function _getEventFilterUrl(event: Event): string | null {
  try {
    let frames;
    try {
      // @ts-ignore we only care about frames if the whole thing here is defined
      frames = event.exception.values[0].stacktrace.frames;
    } catch (e) {
      // ignore
    }
    return frames ? _getLastValidUrl(frames) : null;
  } catch (oO) {
    __DEBUG_BUILD__ &&
      logger.error(
        `Cannot extract url for event ${getEventDescription(event)}`,
      );
    return null;
  }
}
